/*
 * mmaplib.c: Generic routines for File->Char*.
 *
 * Copyright (C) 2000, George Greer <greerga@muc.muohio.edu>
 *
 * Licensed under the GNU General Public License (GPL)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *   or visit ftp://ftp.gnu.org/pub/gnu/COPYING via the Internet.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/fcntl.h>

#include "mmaplib.h"

const char *mmaplib_copyright = "mmaplib.c: Copyright 2000, George Greer <greerga@muc.muohio.edu>";
const char *mmaplib_version = "v0.8 (Build: "__DATE__" "__TIME__")";

/* ----- MMAP Routines -------------------------------------------------------------------------------- */

struct mmap_s {
  int fd;
  size_t size;
  void *map;
};

void *mapopen(const char *filename)
{
  int fd;
  struct stat filestat;
  struct mmap_s *ptrret;
  char *map;

  if ((fd = open(filename, O_RDONLY)) < 0)
    error("open");

  if (fstat(fd, &filestat) < 0)
    error("fstat");

  if (filestat.st_size == 0) {
    fprintf(stderr, "%s is 0 bytes.\n", filename);
    return NULL;
  }

#ifdef WIN32
  map = (char *)calloc(1, filestat.st_size);
  read(fd, map, filestat.st_size);
#else
  if ((map = (char *)mmap(0, filestat.st_size, PROT_READ, MAP_FILE | MAP_SHARED, fd, 0)) == MAP_FAILED)
    error("mmap");
#endif

  ptrret = (struct mmap_s *)calloc(1, sizeof(struct mmap_s));
  ptrret->fd = fd;
  ptrret->size = filestat.st_size;
  ptrret->map = map;

  return ptrret;
}

size_t maplength(void *handle)
{
  return ((struct mmap_s *)handle)->size;
}

const char *mapdata(void *handle)
{
  return ((struct mmap_s *)handle)->map;
}

void mapclose(void *handle)
{
  struct mmap_s *localdata = (struct mmap_s *)handle;

#ifdef WIN32
  free(localdata->map);
#else
  if (munmap(localdata->map, localdata->size) < 0)
    error("munmap");

  if (close(localdata->fd) < 0)
    error("close");
#endif

  free(handle);
}

void error(const char *why)
{
  perror(why);
  exit(1);
}
